################################################################################
# WARNING: This file was autogenerated by create_sync.py. Please do not change
# it and instead make your changes to the aio subpackage and create_sync.py
# as necessary to ensure the implementations to do not diverge.
################################################################################
from types import ModuleType
from typing import Union

from thriftpy2.protocol import (
    TProtocolBase,
    TBinaryProtocolFactory,
)
from thriftpy2.transport import (
    TTransportBase,
    TBufferedTransportFactory,
)
from thriftpy2.thrift import TClient

import httpx

from .transport import THTTPXClient

try:
    from typing import Protocol
except ImportError:
    from typing_extensions import Protocol


class TProtocolFactory(Protocol):
    def get_protocol(self, trans: TTransportBase) -> TProtocolBase:
        ...


class TTransportFactory(Protocol):
    def get_transport(self, trans: TTransportBase) -> TTransportBase:
        ...


def make_client(
    service: ModuleType,
    host: str = 'localhost',
    port: int = 9090,
    url: Union[str, httpx.URL] = None,
    proto_factory: TProtocolFactory = TBinaryProtocolFactory(),
    trans_factory: TTransportFactory = TBufferedTransportFactory(),
    **kwargs,
) -> TClient:
    """
    Create a Thrift client for the given service and transport/protocol
    configuration. Analogous to the socket implementation:
    :py:func:`thriftpy2.rpc.make_client`

    :param service:
        Thrift service (generated module from a .thrift file) to create
        the Thrift client for.
    :param host: Thrift HTTP server hostname
    :param port: Thrift HTTP server port
    :param url: Thrift server URL. Takes precedence over host and port
    :param proto_factory:
        Protocol factory for creating :py:class:`TProtocolBase` instances.
        Must have a ``get_protocol(transport)`` method.
    :param trans_factory:
        Transport factory for creating :py:class:`TTransportBase`
        instances. Must have a ``get_transport(transport)`` method.
    :param kwargs:
        Extra keyword arguments to pass to :py:class:`THTTPXClient`
        which will be passed onto :py:class:`httpx.Client`
    :return: Newly initialized :py:class:`TClient` instance.
    """
    url = url if url else f'http://{host}:{port}/'
    client = THTTPXClient(url, **kwargs)
    transport = trans_factory.get_transport(client)
    protocol = proto_factory.get_protocol(transport)
    transport.open()
    return TClient(service, protocol)
